@@ -49,6 +49,10 @@ typedef void (*HalRunTerminatedCB) (HalDevice *d, guint32 exit_type,
 gboolean
 hald_runner_start_runner(void);
 
+/* Stop the runner daemon */
+void
+hald_runner_stop_runner(void);
+
 /* Start a helper, returns true on a successfull start. 
  * cb will be called on abnormal or premature termination
  * only 
@@ -60,11 +64,19 @@ hald_runner_start (HalDevice *device, const gchar *command_line, char **extra_en
 /* Run a helper program using the commandline, with input as infomation on
  * stdin */
 void
-hald_runner_run(HalDevice *device,
-               const gchar *command_line, char **extra_env, 
-               guint32 timeout,
-               HalRunTerminatedCB cb,
-               gpointer data1, gpointer data2);
+hald_runner_run (HalDevice *device,
+		 const gchar *command_line, char **extra_env, 
+		 guint32 timeout,
+		 HalRunTerminatedCB cb,
+		 gpointer data1, gpointer data2);
+
+void
+hald_runner_run_sync (HalDevice *device,
+		      const gchar *command_line, char **extra_env, 
+		      guint32 timeout,
+		      HalRunTerminatedCB cb,
+		      gpointer data1, gpointer data2);
+
 void
 hald_runner_run_method(HalDevice *device,                                                              
 		       const gchar *command_line, char **extra_env, 
@@ -79,4 +91,7 @@ void hald_runner_kill_all();
 /* called by the core to tell the runner a device was finalized */
 void runner_device_finalized (HalDevice *device);
 
+typedef void (*HaldRunnerRunNotify)(gpointer user_data);
+void hald_runner_set_method_run_notify (HaldRunnerRunNotify cb, gpointer user_data);
+
 #endif 
