@@ -44,6 +44,12 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
+#ifdef HAVE_SYS_INOTIFY_H
+#include <sys/inotify.h>
+#else
+#include "inotify_local.h"
+#endif
+
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 
@@ -54,13 +60,13 @@
 #include "../logger.h"
 #include "../osspec.h"
 #include "../util.h"
+#include "../ids.h"
 
 #include "acpi.h"
 #include "apm.h"
 #include "blockdev.h"
 #include "coldplug.h"
 #include "hotplug.h"
-#include "ids.h"
 #include "pmu.h"
 
 #include "osspec_linux.h"
@@ -225,7 +231,7 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 		   hotplug_event->sysfs.seqnum, action, hotplug_event->sysfs.subsystem, hotplug_event->sysfs.sysfs_path,
 		   hotplug_event->sysfs.device_file, hotplug_event->sysfs.net_ifindex));
 
-	if (strcmp (action, "add") == 0) {
+	if (strcmp (action, "add") == 0 || strcmp (action, "change") == 0) {
 		hotplug_event->action = HOTPLUG_ACTION_ADD;
 		hotplug_event_enqueue (hotplug_event);
 		hotplug_event_process_queue ();
@@ -259,6 +265,130 @@ mount_tree_changed_event (GIOChannel *channel, GIOCondition cond,
 	return TRUE;
 }
 
+void
+osspec_privileged_init (void)
+{
+}
+
+static gboolean
+inotify_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
+{
+	int fd;
+	char buf[256];
+
+	fd = g_io_channel_unix_get_fd (source);
+again:
+	if (read (fd, buf, sizeof (buf)) < 0) {
+		if (errno == EINTR) {
+			goto again;
+		} else {
+			HAL_ERROR (("read: %s", strerror (errno)));
+		}
+	}
+
+	/* TODO: OK, so the right thing to do would be to add watches
+	 * for directories that appear 
+	 */
+
+	osspec_fdi_cache_invalid ();
+	return TRUE;
+}
+
+static gboolean
+add_fdi_watch (int inotify_fd, const char *fdi_dir)
+{
+	int wd;
+	gboolean ret;
+	GDir *dir;
+	GError *error = NULL;
+	const char *name;
+
+	ret = FALSE;
+
+	wd = inotify_add_watch (inotify_fd, fdi_dir, IN_MODIFY | IN_CREATE | IN_DELETE);
+	if (wd < 0) {
+		HAL_ERROR (("Unable to inotify_add_watch() for '%s': %s", fdi_dir, strerror (errno)));
+		goto out;
+	}
+
+	/* watch sub dirs */
+	dir = g_dir_open (fdi_dir, 0, &error);
+	if (dir == NULL)
+		goto out;
+	while ((name = g_dir_read_name (dir)) != NULL) {
+		char *sub_dir;
+
+		sub_dir = g_strdup_printf ("%s/%s", fdi_dir, name);
+		if (g_file_test (sub_dir, G_FILE_TEST_IS_DIR)) {
+			add_fdi_watch (inotify_fd, sub_dir);
+		}
+		g_free (sub_dir);
+	}
+	g_dir_close (dir);
+
+	ret = TRUE;
+out:
+	return ret;
+}
+
+static void
+watch_fdi_files (void)
+{
+	int inotify_fd;
+	char *hal_fdi_source_preprobe;
+	char *hal_fdi_source_information;
+	char *hal_fdi_source_policy;
+	GIOChannel *inotify_channel;
+
+	hal_fdi_source_preprobe = getenv ("HAL_FDI_SOURCE_PREPROBE");
+	hal_fdi_source_information = getenv ("HAL_FDI_SOURCE_INFORMATION");
+	hal_fdi_source_policy = getenv ("HAL_FDI_SOURCE_POLICY");
+
+	inotify_fd = inotify_init ();
+	if (inotify_fd < 0) {
+		DIE (("Unable to initialize inotify: %s ", strerror (errno)));
+		goto out;
+	}
+	inotify_channel = g_io_channel_unix_new (inotify_fd);
+	g_io_add_watch (inotify_channel, G_IO_IN, inotify_data, NULL);
+	g_io_channel_unref (inotify_channel);
+
+
+	if (hal_fdi_source_preprobe != NULL) {
+		if (!add_fdi_watch (inotify_fd, hal_fdi_source_preprobe))
+			goto out;
+	} else {
+		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/preprobe"))
+			goto out;
+		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/preprobe"))
+			goto out;
+	}
+
+	if (hal_fdi_source_information != NULL) {
+		if (!add_fdi_watch (inotify_fd, hal_fdi_source_information))
+			goto out;
+	} else {
+		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/information"))
+			goto out;
+		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/information"))
+			goto out;
+	}
+
+	if (hal_fdi_source_policy != NULL) {
+		if (!add_fdi_watch (inotify_fd, hal_fdi_source_policy))
+			goto out;
+	} else {
+		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/policy"))
+			goto out;
+		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/policy"))
+			goto out;
+	}
+
+	return;
+out:
+	DIE (("Error watching fdi files"));
+}
+
 void
 osspec_init (void)
 {
@@ -320,19 +450,19 @@ osspec_init (void)
 	 *Load various hardware id databases
 	 */
 	ids_init ();
+
+	/* watch fdi directories */
+	watch_fdi_files ();	
 }
 
 static void 
 computer_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
 {
-	HAL_INFO (("Add callouts completed udi=%s", d->udi));
+	HAL_INFO (("Add callouts completed udi=%s", hal_device_get_udi (d)));
 
 	/* Move from temporary to global device store */
 	hal_device_store_remove (hald_get_tdl (), d);
 	hal_device_store_add (hald_get_gdl (), d);
-
-	/* start processing events */
-	hotplug_event_process_queue ();
 }
 
 void
@@ -346,6 +476,10 @@ hotplug_queue_now_empty (void)
 static void
 computer_probing_helper_done (HalDevice *d)
 {
+	/* if not set, set a default value */
+	if (!hal_device_has_property (d, "system.formfactor")) {
+		hal_device_property_set_string (d, "system.formfactor", "unknown");
+	}
 	di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
 	di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
 
@@ -363,17 +497,15 @@ computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type,
 	const char *system_version;
 
 	if (exit_type == HALD_RUN_FAILED) {
-		/* set a default value */
-		if (!hal_device_has_property (d, "system.formfactor"))
-			hal_device_property_set_string (d, "system.formfactor", "unknown");
 		goto out;
 	}
 
-	if ((system_manufacturer = hal_device_property_get_string (d, "smbios.system.manufacturer")) != NULL &&
-	    (system_product = hal_device_property_get_string (d, "smbios.system.product")) != NULL &&
-	    (system_version = hal_device_property_get_string (d, "smbios.system.version")) != NULL) {
+	if ((system_manufacturer = hal_device_property_get_string (d, "system.hardware.vendor")) != NULL &&
+	    (system_product = hal_device_property_get_string (d, "system.hardware.product")) != NULL &&
+	    (system_version = hal_device_property_get_string (d, "system.hardware.version")) != NULL) {
 		char buf[128];
 
+		/* depricated 2008-02-28 */
 		hal_device_property_set_string (d, "system.vendor", system_manufacturer);
 
 		if (strcmp(system_version, "Not Specified" ) != 0 ) {
@@ -433,10 +565,7 @@ computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type,
 				}
 			}
 		       
-		} else {
-			/* set a default value */
-			hal_device_property_set_string (d, "system.formfactor", "unknown");
-		}
+		} 
 	}
 out:
 	computer_probing_helper_done (d);
@@ -450,6 +579,7 @@ set_suspend_hibernate_keys (HalDevice *d)
 	ssize_t read;
 	size_t len;
 	char *poweroptions;
+	const char *pmtype;
 	FILE *fp;
 
 	can_suspend = FALSE;
@@ -475,6 +605,12 @@ set_suspend_hibernate_keys (HalDevice *d)
 		can_hibernate = TRUE;
 	free (poweroptions);
 
+	pmtype = hal_device_property_get_string (d, "power_management.type");
+	if (pmtype != NULL && strcmp(pmtype, "pmu") == 0) {
+		/* We got our own helper for suspend PMU machines */
+		can_suspend = TRUE;
+	}
+
 	/* check for the presence of suspend2 */
 	if (access ("/proc/software_suspend", F_OK) == 0)
 		can_hibernate = TRUE;
@@ -484,6 +620,7 @@ set_suspend_hibernate_keys (HalDevice *d)
 		can_hibernate = TRUE;
 out:
 	hal_device_property_set_bool (d, "power_management.can_suspend", can_suspend);
+	hal_device_property_set_bool (d, "power_management.can_suspend_hybrid", FALSE);
 	hal_device_property_set_bool (d, "power_management.can_hibernate", can_hibernate);
 
 	/* WARNING: These keys are depreciated and power_management.can_suspend
@@ -493,18 +630,108 @@ out:
 	hal_device_property_set_bool (d, "power_management.can_suspend_to_disk", can_hibernate);
 }
 
+static void
+get_openfirmware_entry(HalDevice *d, char *property, char *entry, 
+                       gboolean multivalue) 
+{
+	char *contents;
+	gsize length;
+	if (!g_file_get_contents(entry, &contents, &length, NULL)) {
+		return;
+	}
+	if (multivalue) {
+		gsize offset = 0;
+		while (offset < length) { 
+			hal_device_property_strlist_append(d, property, contents + offset);
+			for (; offset < length - 1 && contents[offset] != '\0'; offset++)
+				;
+			offset++;
+		}
+	} else {
+		hal_device_property_set_string(d, property, contents);
+	}
+	free(contents);
+}
+
+static void
+detect_openfirmware_formfactor(HalDevice *root) 
+{
+	int x;
+	struct { gchar *model; gchar *formfactor; } model_formfactor[] =
+		{ 
+			{ "RackMac"   , "server" },
+			{ "AAPL,3400" , "laptop"  },
+			{ "AAPL,3500" , "laptop"  },
+			{ "PowerBook" , "laptop"  },
+			{ "AAPL"      , "desktop" },
+			{ "iMac"      , "desktop" },
+			{ "PowerMac"  , "desktop" },
+			{ "Pegasos"   , "desktop" },
+			{NULL, NULL }
+		};
+	const gchar *model =
+	  hal_device_property_get_string(root, "openfirmware.model");
+	if (model == NULL) 
+		return;
+	for (x = 0 ; model_formfactor[x].model ; x++) {
+		if (strstr(model, model_formfactor[x].model)) {
+			hal_device_property_set_string (root, "system.formfactor",
+				model_formfactor[x].formfactor);
+			break;
+		}
+	}
+}
+
+static void
+probe_openfirmware(HalDevice *root) 
+{
+#define DEVICE_TREE "/proc/device-tree/"
+	if (!g_file_test(DEVICE_TREE, G_FILE_TEST_IS_DIR)) {
+		return;
+	}
+	get_openfirmware_entry(root, "openfirmware.model", 
+		DEVICE_TREE "model", FALSE);
+	get_openfirmware_entry(root, "openfirmware.compatible", 
+		DEVICE_TREE "compatible", TRUE);
+	detect_openfirmware_formfactor(root);
+}
+
+static void
+decode_dmi (HalDevice *d)
+{
+        if (g_file_test ("/usr/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+            g_file_test ("/bin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+            g_file_test ("/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+            g_file_test ("/usr/local/sbin//dmidecode", G_FILE_TEST_IS_EXECUTABLE)) {
+                hald_runner_run (d, "hald-probe-smbios", NULL, HAL_HELPER_TIMEOUT,
+                                 computer_probing_pcbios_helper_done, NULL, NULL);
+        } else {
+                /* no probing */
+                probe_openfirmware (d);
+                computer_probing_helper_done (d);
+        }
+
+}
+
+static void 
+computer_probing_pm_is_supported_helper_done (HalDevice *d, guint32 exit_type, 
+                                              gint return_code, gchar **error, 
+                                              gpointer data1, gpointer data2)
+{
+        HAL_INFO (("In computer_probing_pm_is_supported_helper_done"));
+        decode_dmi (d);
+}
+
 void 
 osspec_probe (void)
 {
 	HalDevice *root;
 	struct utsname un;
-	gboolean should_decode_dmi;
-
-	should_decode_dmi = FALSE;
 
+	hald_runner_set_method_run_notify ((HaldRunnerRunNotify) hotplug_event_process_queue, NULL);
 	root = hal_device_new ();
+	hal_device_property_set_string (root, "info.subsystem", "unknown");
 	hal_device_property_set_string (root, "info.bus", "unknown");
-	hal_device_property_set_string (root, "linux.sysfs_path_device", "(none)");
 	hal_device_property_set_string (root, "info.product", "Computer");
 	hal_device_property_set_string (root, "info.udi", "/org/freedesktop/Hal/devices/computer");
 	hal_device_set_udi (root, "/org/freedesktop/Hal/devices/computer");
@@ -525,12 +752,10 @@ osspec_probe (void)
 	HAL_INFO (("Synthesizing powermgmt events..."));
 	if (acpi_synthesize_hotplug_events ()) {
 		HAL_INFO (("ACPI capabilities found"));
-		should_decode_dmi = TRUE;
 	} else if (pmu_synthesize_hotplug_events ()) {
 		HAL_INFO (("PMU capabilities found"));		
 	} else if (apm_synthesize_hotplug_events ()) {
 		HAL_INFO (("APM capabilities found"));		
-		should_decode_dmi = TRUE;
 	} else {
 		HAL_INFO (("No powermgmt capabilities"));		
 	}
@@ -544,16 +769,14 @@ osspec_probe (void)
 	 */
 	set_suspend_hibernate_keys (root);
 
-	/* TODO: add prober for PowerMac's */
-	if (should_decode_dmi) {
-		hald_runner_run (root, "hald-probe-smbios", NULL, HAL_HELPER_TIMEOUT,
-        	                 computer_probing_pcbios_helper_done, NULL, NULL);
-	} else {
-		/* set a default value, can be overridden by others */
-		hal_device_property_set_string (root, "system.formfactor", "unknown");
-		/* no probing */
-		computer_probing_helper_done (root);
-  	}
+        /* Try and set the suspend/hibernate keys using pm-is-supported
+         */
+        if (g_file_test ("/usr/bin/pm-is-supported", G_FILE_TEST_IS_EXECUTABLE)) {
+                hald_runner_run (root, "hal-system-power-pm-is-supported", NULL, HAL_HELPER_TIMEOUT,
+                                 computer_probing_pm_is_supported_helper_done, NULL, NULL);
+        } else {
+                decode_dmi (root);
+        }
 }
 
 DBusHandlerResult
@@ -606,40 +829,79 @@ hal_util_set_driver (HalDevice *d, const char *property_name, const char *sysfs_
 	return ret;
 }
 
-/** Find the closest ancestor by looking at sysfs paths
- *
- *  @param  sysfs_path           Path into sysfs, e.g. /sys/devices/pci0000:00/0000:00:1d.7/usb1/1-0:1.0
- *  @return                      Parent Hal Device Object or #NULL if there is none
- */
-HalDevice *
-hal_util_find_closest_ancestor (const gchar *sysfs_path)
-{	
-	gchar buf[512];
-	HalDevice *parent;
-
-	parent = NULL;
-
-	strncpy (buf, sysfs_path, sizeof (buf));
-	do {
-		char *p;
-
-		p = strrchr (buf, '/');
-		if (p == NULL)
-			break;
-		*p = '\0';
-
-		parent = hal_device_store_match_key_value_string (hald_get_gdl (), 
-								  "linux.sysfs_path_device", 
-								  buf);
-		if (parent != NULL)
-			break;
-
-	} while (TRUE);
-
-	return parent;
+static gboolean get_parent_device(char *path)
+{
+	/* go up one directory */
+	if (!hal_util_path_ascend (path))
+		return FALSE;
+	if (g_str_has_suffix (path, "/class"))
+		return FALSE;
+	if (g_str_has_suffix (path, "/block"))
+		return FALSE;
+	if (g_str_has_suffix (path, "/devices"))
+		return FALSE;
+	return TRUE;
 }
 
-void 
+/* return the first already known parent device */
+gboolean
+hal_util_find_known_parent (const gchar *sysfs_path, HalDevice **parent, gchar **parent_path)
+{
+	gchar *target;
+	HalDevice *parent_dev = NULL;
+	gchar *parent_devpath;
+	char parentdevpath[HAL_PATH_MAX];
+	gboolean retval = FALSE;
+
+	parent_devpath = g_strdup (sysfs_path);
+	while (TRUE) {
+		if (!get_parent_device (parent_devpath))
+			break;
+
+		parent_dev = hal_device_store_match_key_value_string (hald_get_gdl (),
+								      "linux.sysfs_path",
+								      parent_devpath);
+		if (parent_dev != NULL)
+			goto out;
+	}
+	g_free (parent_devpath);
+	parent_devpath = NULL;
+
+	/* try if the parent chain is constructed by the device-link */
+	g_snprintf (parentdevpath, HAL_PATH_MAX, "%s/device", sysfs_path);
+	if ((target = hal_util_readlink (parentdevpath)) != NULL) {
+		parent_devpath = hal_util_get_normalized_path (sysfs_path, target);
+
+		while (TRUE) {
+			parent_dev = hal_device_store_match_key_value_string (hald_get_gdl (),
+									      "linux.sysfs_path",
+									      parent_devpath);
+			if (parent_dev != NULL)
+				goto out;
+
+			/* go up one directory */
+			if (!get_parent_device (parent_devpath))
+				break;
+		}
+		g_free (parent_devpath);
+		parent_devpath = NULL;
+	}
+
+out:
+	if (parent_dev != NULL) {
+		HAL_INFO (("hal_util_find_known_parent: '%s'->'%s'", sysfs_path, parent_devpath));
+		retval = TRUE;
+	}
+	if (parent != NULL)
+		*parent = parent_dev;
+	if (parent_path != NULL)
+		*parent_path = parent_devpath;
+	else
+		g_free (parent_devpath);
+	return retval;
+}
+
+void
 osspec_refresh_mount_state_for_block_device (HalDevice *d)
 {
 	blockdev_refresh_mount_state (d);
