@@ -32,6 +32,9 @@
 
 #include "device.h"
 
+/** Initialize the kernel specific parts of the daemon, as root */
+void osspec_privileged_init (void);
+
 /** Initialize the kernel specific parts of the daemon */
 void osspec_init (void);
 
@@ -48,6 +51,9 @@ gboolean osspec_device_reprobe (HalDevice *d);
 /* Called to refresh mount state for a device object of capability volume */
 void osspec_refresh_mount_state_for_block_device (HalDevice *d);
 
+/* Called by OS specific code to tell that the fdi cache is invalid */
+void osspec_fdi_cache_invalid (void);
+
 /** Called when the org.freedesktop.Hal service receives a messaged that the generic daemon 
  *  doesn't handle. Can be used for intercepting messages from kernel or core OS components.
  *
