@@ -54,6 +54,26 @@
 
 #include "util.h"
 
+/**
+ * Determine wether the given character is valid as the first character
+ * in a name.
+ */
+#define VALID_INITIAL_NAME_CHARACTER(c)         \
+  ( ((c) >= 'A' && (c) <= 'Z') ||               \
+    ((c) >= 'a' && (c) <= 'z') ||               \
+    ((c) == '_') )
+
+/**
+ * Determine wether the given character is valid as a second or later
+ * character in a name
+ */
+#define VALID_NAME_CHARACTER(c)                 \
+  ( ((c) >= '0' && (c) <= '9') ||               \
+    ((c) >= 'A' && (c) <= 'Z') ||               \
+    ((c) >= 'a' && (c) <= 'z') ||               \
+    ((c) == '_') )
+ 
+
 gboolean 
 hal_util_remove_trailing_slash (gchar *path)
 {
@@ -403,6 +423,25 @@ hal_util_set_string_from_file (HalDevice *d, const gchar *key, const gchar *dire
 	return ret;
 }
 
+gboolean
+hal_util_set_double_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file)
+{
+	double value;
+	gchar *buf, *end;
+	gboolean ret;
+
+	ret = FALSE;
+
+	if ((buf = hal_util_get_string_from_file (directory, file)) != NULL) {
+		value = strtod(buf, &end);
+		if (errno != ERANGE) {
+			ret = hal_device_property_set_double (d, key, value);
+		}
+	}
+	
+	return ret;
+}
+
 void
 hal_util_compute_udi (HalDeviceStore *store, gchar *dst, gsize dstsize, const gchar *format, ...)
 {
@@ -459,22 +498,21 @@ hal_util_grep_discard_existing_data (void)
 	_grep_can_reuse = FALSE;
 }
 
-/** Given a directory and filename, open the file and search for the
+/**  
+ *  hal_util_grep_file:
+ *  @directory:          Directory, e.g. "/proc/acpi/battery/BAT0"
+ *  @file:               File, e.g. "info"
+ *  @linestart:          Start of line, e.g. "serial number"
+ *  @reuse:              Whether we should reuse the file contents if the file is the same; can be 
+ *                       cleared with hal_util_grep_discard_existing_data()
+ *  Returns:             NULL if not found, otherwise the remainder of the line, e.g. 
+ *                       ":           21805" if the file /proc/acpi/battery/BAT0 contains
+ *                       this line "serial number:           21805". The string is only valid 
+ *                       until the next invocation of this function.
+ *
+ *  Given a directory and filename, open the file and search for the
  *  first line that starts with the given linestart string. Returns
  *  the rest of the line as a string if found.
- *
- *  @param  directory           Directory, e.g. "/proc/acpi/battery/BAT0"
- *  @param  file                File, e.g. "info"
- *  @param  linestart           Start of line, e.g. "serial number"
- *  @param  reuse               Whether we should reuse the file contents
- *                              if the file is the same; can be cleared
- *                              with hal_util_grep_discard_existing_data()
- *  @return                     NULL if not found, otherwise the remainder
- *                              of the line, e.g. ":           21805" if
- *                              the file /proc/acpi/battery/BAT0 contains
- *                              this line "serial number:           21805"
- *                              The string is only valid until the next
- *                              invocation of this function.
  */
 gchar *
 hal_util_grep_file (const gchar *directory, const gchar *file, const gchar *linestart, gboolean reuse)
@@ -548,6 +586,101 @@ out:
 	return result;
 }
 
+/**  
+ *  hal_util_grep_file_next_line:
+ *  @directory:          Directory, e.g. "/proc/acpi/battery/BAT0"
+ *  @file:               File, e.g. "info"
+ *  @linestart:          Start of line, e.g. "serial number"
+ *  @reuse:              Whether we should reuse the file contents if the file is the same; can be 
+ *                       cleared with hal_util_grep_discard_existing_data()
+ *  Returns:             NULL if not found, otherwise the next line. The string is only valid 
+ *                       until the next invocation of this function.
+ *
+ *  Given a directory and filename, open the file and search for the
+ *  first line that starts with the given linestart string. Returns
+ *  the next line as a string if found.
+ */
+gchar *
+hal_util_grep_file_next_line (const gchar *directory, const gchar *file, const gchar *linestart, gboolean reuse)
+{
+	static gchar buf[2048];
+	static unsigned int bufsize;
+	static gchar filename[HAL_PATH_MAX];
+	static gchar oldfilename[HAL_PATH_MAX];
+	gchar *result;
+	gsize linestart_len;
+	gchar *p;
+
+	result = NULL;
+
+	/* TODO: use reuse and _grep_can_reuse parameters to avoid loading
+	 *       the file again and again
+	 */
+
+	if (file != NULL && strlen (file) > 0)
+		snprintf (filename, sizeof (filename), "%s/%s", directory, file);
+	else
+		strncpy (filename, directory, sizeof (filename));
+
+	if (_grep_can_reuse && reuse && strcmp (oldfilename, filename) == 0) {
+		/* just reuse old file; e.g. bufsize, buf */
+		/*HAL_INFO (("hal_util_grep_file: reusing buf for %s", filename));*/
+	} else {
+		FILE *f;
+
+		f = fopen (filename, "r");
+		if (f == NULL)
+			goto out;
+		bufsize = fread (buf, sizeof (char), sizeof (buf) - 1, f);
+		buf[bufsize] = '\0';
+		fclose (f);
+
+		/*HAL_INFO (("hal_util_grep_file: read %s of %d bytes", filename, bufsize));*/
+	}
+
+	/* book keeping */
+	_grep_can_reuse = TRUE;
+	strncpy (oldfilename, filename, sizeof(oldfilename));
+
+	linestart_len = strlen (linestart);
+
+	/* analyze buf */
+	p = buf;
+	do {
+		unsigned int linelen;
+		static char line[256];
+
+		for (linelen = 0; p[linelen] != '\n' && p[linelen] != '\0'; linelen++)
+			;
+
+		if (linelen < sizeof (line)) {
+
+			strncpy (line, p, linelen);
+			line[linelen] = '\0';
+
+			if (strncmp (line, linestart, linestart_len) == 0) {
+				/* go to next line */
+				p += linelen + 1;
+				if (p < buf + bufsize) { /* if there is one*/
+					for (linelen = 0; p[linelen] != '\n' && p[linelen] != '\0'; linelen++)
+						;
+					strncpy (line, p, linelen);
+					line[linelen] = '\0';
+
+					result = line;
+				}
+				goto out;
+			}
+		}
+
+		p += linelen + 1;
+
+	} while (p < buf + bufsize);
+
+out:
+	return result;
+}
+
 gchar *
 hal_util_grep_string_elem_from_file (const gchar *directory, const gchar *file, 
 				     const gchar *linestart, guint elem, gboolean reuse)
@@ -564,6 +697,20 @@ hal_util_grep_string_elem_from_file (const gchar *directory, const gchar *file,
 	if (((line = hal_util_grep_file (directory, file, linestart, reuse)) == NULL) || (strlen (line) == 0))
 		goto out;
 
+	/* this means take all elements, e.g. "DELL Y7637" */
+	if (elem == G_MAXUINT) {
+		/* just a null string ": " we want to discard */
+		if (strlen (line) < 2) {
+			res = NULL;
+			goto out;
+		}
+		/* strip leading spaces, missing out the ":" first char */
+		line = g_strchug (line + 1);
+		strncpy (buf, line, sizeof (buf));
+		res = buf;
+		goto out;
+	}
+
 	tokens = g_strsplit_set (line, " \t:", 0);
 	for (i = 0, j = 0; tokens[i] != NULL; i++) {
 		if (strlen (tokens[i]) == 0)
@@ -579,7 +726,6 @@ hal_util_grep_string_elem_from_file (const gchar *directory, const gchar *file,
 out:
 	if (tokens != NULL)
 		g_strfreev (tokens);
-
 	return res;
 }
 
@@ -607,7 +753,21 @@ out:
 	return value;
 }
 
-/** Get a string value from a formatted text file and assign it to
+/** 
+ *  hal_util_set_string_elem_from_file:  
+ *  @d:                  Device object
+ *  @key:                Property name
+ *  @directory:          Directory, e.g. "/proc/acpi/battery/BAT0"
+ *  @file:               File, e.g. "info"
+ *  @linestart:          Start of line, e.g. "design voltage"
+ *  @elem:               Element number after linestart to extract
+ *                       excluding whitespace and ':' characters.
+ *			 If set to G_MAXUINT then all elements are set
+ *
+ *  Returns:             TRUE, if, and only if, the value could be
+ *                       extracted and the property was set
+ *
+ *  Get a string value from a formatted text file and assign it to
  *  a property on a device object.
  *
  *  Example: Given that the file /proc/acpi/battery/BAT0/info contains
@@ -618,16 +778,6 @@ out:
  *  then hal_util_set_string_elem_from_file (d, "battery.foo",
  *  "/proc/acpi/battery/BAT0", "info", "design voltage", 1) will assign
  *  the string "mV" to the property "battery.foo" on d.
- *
- *  @param  d                   Device object
- *  @param  key                 Property name
- *  @param  directory           Directory, e.g. "/proc/acpi/battery/BAT0"
- *  @param  file                File, e.g. "info"
- *  @param  linestart           Start of line, e.g. "design voltage"
- *  @param  elem                Element number after linestart to extract
- *                              excluding whitespace and ':' characters.
- *  @return                     TRUE, if, and only if, the value could be
- *                              extracted and the property was set
  */
 gboolean
 hal_util_set_string_elem_from_file (HalDevice *d, const gchar *key, 
@@ -647,7 +797,20 @@ out:
 	return res;
 }
 
-/** Get an integer value from a formatted text file and assign it to
+/** 
+ *  hal_util_set_int_elem_from_file: 
+ *  @d:                  Device object
+ *  @key:                Property name
+ *  @directory:          Directory, e.g. "/proc/acpi/battery/BAT0"
+ *  @file:               File, e.g. "info"
+ *  @linestart:          Start of line, e.g. "design voltage"
+ *  @elem :              Element number after linestart to extract
+ *                       excluding whitespace and ':' characters.
+ *
+ *  Returns:             TRUE, if, and only if, the value could be
+ *                       extracted and the property was set
+ *
+ *  Get an integer value from a formatted text file and assign it to
  *  a property on a device object.
  *
  *  Example: Given that the file /proc/acpi/battery/BAT0/info contains
@@ -658,16 +821,6 @@ out:
  *  then hal_util_set_int_elem_from_file (d, "battery.foo",
  *  "/proc/acpi/battery/BAT0", "info", "design voltage", 0) will assign
  *  the integer 10800 to the property "battery.foo" on d.
- *
- *  @param  d                   Device object
- *  @param  key                 Property name
- *  @param  directory           Directory, e.g. "/proc/acpi/battery/BAT0"
- *  @param  file                File, e.g. "info"
- *  @param  linestart           Start of line, e.g. "design voltage"
- *  @param  elem                Element number after linestart to extract
- *                              excluding whitespace and ':' characters.
- *  @return                     TRUE, if, and only if, the value could be
- *                              extracted and the property was set
  */
 gboolean
 hal_util_set_int_elem_from_file (HalDevice *d, const gchar *key, 
@@ -696,7 +849,21 @@ out:
 
 }
 
-/** Get a value from a formatted text file, test it against a given
+/**  
+ *  hal_util_set_bool_elem_from_file:
+ *  @d:                  Device object
+ *  @key:                Property name
+ *  @directory:          Directory, e.g. "/proc/acpi/battery/BAT0"
+ *  @file:               File, e.g. "info"
+ *  @linestart:          Start of line, e.g. "design voltage"
+ *  @elem:               Element number after linestart to extract
+ *                       excluding whitespace and ':' characters.
+ *  @expected:           Value to test against
+ *
+ *  Returns:             TRUE, if, and only if, the value could be
+ *                       extracted and the property was set
+ *
+ *  Get a value from a formatted text file, test it against a given
  *  value, and set a boolean property on a device object with the
  *  test result.
  *
@@ -714,17 +881,6 @@ out:
  *    "present:                 no"
  *
  *  the value assigned will be FALSE.
- *
- *  @param  d                   Device object
- *  @param  key                 Property name
- *  @param  directory           Directory, e.g. "/proc/acpi/battery/BAT0"
- *  @param  file                File, e.g. "info"
- *  @param  linestart           Start of line, e.g. "design voltage"
- *  @param  elem                Element number after linestart to extract
- *                              excluding whitespace and ':' characters.
- *  @param  expected            Value to test against
- *  @return                     TRUE, if, and only if, the value could be
- *                              extracted and the property was set
  */
 gboolean
 hal_util_set_bool_elem_from_file (HalDevice *d, const gchar *key, 
@@ -829,19 +985,21 @@ callout_do_next (Callout *c)
 		g_strfreev (c->extra_env);
 		g_free (c);
 
-		callback (d, userdata1, userdata2);
+		if (callback != NULL) {
+			callback (d, userdata1, userdata2);
+		}
 
 	} else {
-    hald_runner_run(c->d, c->programs[c->next_program], c->extra_env,
-                    HAL_HELPER_TIMEOUT, callout_terminated,
-                    (gpointer)c, NULL);
+		hald_runner_run(c->d, c->programs[c->next_program], c->extra_env,
+				HAL_HELPER_TIMEOUT, callout_terminated,
+				(gpointer)c, NULL);
 		c->next_program++;
 	}
 }
 
-static void
+void
 hal_callout_device (HalDevice *d, HalCalloutsDone callback, gpointer userdata1, gpointer userdata2, 
-		    GSList *programs, gchar **extra_env)
+		    char **programs, gchar **extra_env)
 {
 	Callout *c;
 
@@ -850,7 +1008,7 @@ hal_callout_device (HalDevice *d, HalCalloutsDone callback, gpointer userdata1,
 	c->callback = callback;
 	c->userdata1 = userdata1;
 	c->userdata2 = userdata2;
-	c->programs = hal_util_dup_strv_from_g_slist (programs);
+	c->programs = programs;
 	c->extra_env = g_strdupv (extra_env);
 	c->next_program = 0;
 
@@ -860,15 +1018,17 @@ hal_callout_device (HalDevice *d, HalCalloutsDone callback, gpointer userdata1,
 void
 hal_util_callout_device_add (HalDevice *d, HalCalloutsDone callback, gpointer userdata1, gpointer userdata2)
 {
-	GSList *programs;
+	char **programs;
 	gchar *extra_env[2] = {"HALD_ACTION=add", NULL};
 
-	if ((programs = hal_device_property_get_strlist (d, "info.callouts.add")) == NULL) {
+	programs = hal_device_property_dup_strlist_as_strv (d, "info.callouts.add");
+	if (programs == NULL) {
 		callback (d, userdata1, userdata2);
 		goto out;
 	}	
 
-	HAL_INFO (("Add callouts for udi=%s", d->udi));
+
+	HAL_INFO (("Add callouts for udi=%s", hal_device_get_udi (d)));
 
 	hal_callout_device (d, callback, userdata1, userdata2, programs, extra_env);
 out:
@@ -878,15 +1038,16 @@ out:
 void
 hal_util_callout_device_remove (HalDevice *d, HalCalloutsDone callback, gpointer userdata1, gpointer userdata2)
 {
-	GSList *programs;
+	char **programs;
 	gchar *extra_env[2] = {"HALD_ACTION=remove", NULL};
 
-	if ((programs = hal_device_property_get_strlist (d, "info.callouts.remove")) == NULL) {
+	programs = hal_device_property_dup_strlist_as_strv (d, "info.callouts.remove");
+	if (programs == NULL) {
 		callback (d, userdata1, userdata2);
 		goto out;
 	}	
 
-	HAL_INFO (("Remove callouts for udi=%s", d->udi));
+	HAL_INFO (("Remove callouts for udi=%s", hal_device_get_udi (d)));
 
 	hal_callout_device (d, callback, userdata1, userdata2, programs, extra_env);
 out:
@@ -896,15 +1057,16 @@ out:
 void
 hal_util_callout_device_preprobe (HalDevice *d, HalCalloutsDone callback, gpointer userdata1, gpointer userdata2)
 {
-	GSList *programs;
+	char **programs;
 	gchar *extra_env[2] = {"HALD_ACTION=preprobe", NULL};
 
-	if ((programs = hal_device_property_get_strlist (d, "info.callouts.preprobe")) == NULL) {
+	programs = hal_device_property_dup_strlist_as_strv (d, "info.callouts.preprobe");
+	if (programs == NULL) {
 		callback (d, userdata1, userdata2);
 		goto out;
 	}	
 
-	HAL_INFO (("Preprobe callouts for udi=%s", d->udi));
+	HAL_INFO (("Preprobe callouts for udi=%s", hal_device_get_udi (d)));
 
 	hal_callout_device (d, callback, userdata1, userdata2, programs, extra_env);
 out:
@@ -928,10 +1090,12 @@ hal_util_strdup_valid_utf8 (const char *str)
 		count++;
 	}
 	
-	if (strlen(newstr) == count)
+	if (strlen(newstr) == count) {
+		g_free (newstr);
 		return NULL;
-	else
+	} else {
 		return newstr;
+	}
 }
 
 void
@@ -1055,6 +1219,8 @@ hal_util_is_mounted_by_hald (const char *mount_point)
 		g_strfreev (line_elements);
 	}
 
+	g_strfreev (lines);
+
 out:
 	if (hal_mtab != NULL)
 		fclose (hal_mtab);
@@ -1065,3 +1231,55 @@ out:
 }
 
 
+static gchar path_buffer [HAL_PATH_MAX];
+
+char *
+hal_util_readlink (const char *link)
+{
+	memset (path_buffer, 0, HAL_PATH_MAX);
+	if(readlink(link, path_buffer, HAL_PATH_MAX-1) < 0)
+		return NULL;
+	
+	return path_buffer;
+}
+
+gboolean
+is_valid_interface_name (const char *name) {
+
+	const char *end;
+	const char *last_dot;
+
+	last_dot = NULL;
+
+	if (strlen(name) == 0)
+		return FALSE;	
+
+	end = name + strlen(name);
+
+	if (*name == '.') /* disallow starting with a . */
+		return FALSE;
+	else if (!VALID_INITIAL_NAME_CHARACTER (*name))
+		return FALSE;
+  	else
+    		++name;
+
+  	while (name != end) {
+      		if (*name == '.') {
+          		if ((name + 1) == end)
+            			return FALSE;
+          		else if (!VALID_INITIAL_NAME_CHARACTER (*(name + 1)))
+            			return FALSE;
+          		last_dot = name;
+          		++name; /* we just validated the next char, so skip two */
+        	}
+      		else if (!VALID_NAME_CHARACTER (*name)) {
+          		return FALSE;
+        	}
+      		++name;
+    	}
+	if (last_dot == NULL)
+		return FALSE;
+
+  	return TRUE;
+}
+
