@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: libhal-storage.h,v 1.17 2006/02/23 00:23:13 david Exp $
+ * CVSID: $Id$
  *
  * libhal-storage.h : HAL convenience library for storage devices and volumes
  *
@@ -32,22 +32,17 @@
 
 #if defined(__cplusplus)
 extern "C" {
+#if 0
+} /* shut up emacs indenting */
+#endif
 #endif
 
-/**
- * @addtogroup HAL Storage and Volume abstraction
- *
- * @{
- */
-
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
 struct LibHalDrive_s;
 typedef struct LibHalDrive_s LibHalDrive;
 struct LibHalVolume_s;
 typedef struct LibHalVolume_s LibHalVolume;
 struct LibHalStoragePolicy_s;
 typedef struct LibHalStoragePolicy_s LibHalStoragePolicy;
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
 
 typedef enum {
@@ -128,7 +123,10 @@ typedef enum {
 	LIBHAL_STORAGE_ICON_DISC_BDRE                      = 0x3000c,
 	LIBHAL_STORAGE_ICON_DISC_HDDVDROM                  = 0x3000d,
 	LIBHAL_STORAGE_ICON_DISC_HDDVDR                    = 0x3000e,
-	LIBHAL_STORAGE_ICON_DISC_HDDVDRW                   = 0x3000f
+	LIBHAL_STORAGE_ICON_DISC_HDDVDRW                   = 0x3000f,
+	LIBHAL_STORAGE_ICON_DISC_MRW                       = 0x30010,
+	LIBHAL_STORAGE_ICON_DISC_MRWW                      = 0x30011,
+	LIBHAL_STORAGE_ICON_DISC_MO                        = 0x30012
 } LibHalStoragePolicyIcon;
 
 typedef struct {
@@ -136,17 +134,17 @@ typedef struct {
 	const char *icon_path;
 } LibHalStoragePolicyIconPair;
 
-LibHalStoragePolicy *libhal_storage_policy_new		    (void);
-void                 libhal_storage_policy_free		    (LibHalStoragePolicy *policy);
+LibHalStoragePolicy *libhal_storage_policy_new		    (void) LIBHAL_DEPRECATED;
+void                 libhal_storage_policy_free		    (LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 
 void                 libhal_storage_policy_set_icon_path    (LibHalStoragePolicy *policy, 
 		   					     LibHalStoragePolicyIcon icon,
-							     const char *path);
+							     const char *path) LIBHAL_DEPRECATED;
 
 void                 libhal_storage_policy_set_icon_mapping (LibHalStoragePolicy *policy, 
-							     LibHalStoragePolicyIconPair *pairs);
+							     LibHalStoragePolicyIconPair *pairs) LIBHAL_DEPRECATED;
 const char  	    *libhal_storage_policy_lookup_icon	    (LibHalStoragePolicy *policy, 
-						  	     LibHalStoragePolicyIcon icon);
+						  	     LibHalStoragePolicyIcon icon) LIBHAL_DEPRECATED;
 
 typedef enum {
 	LIBHAL_DRIVE_BUS_UNKNOWN     = 0x00,
@@ -171,26 +169,31 @@ typedef enum {
 	LIBHAL_DRIVE_TYPE_PORTABLE_AUDIO_PLAYER = 0x0a,
 	LIBHAL_DRIVE_TYPE_ZIP                   = 0x0b,
 	LIBHAL_DRIVE_TYPE_JAZ                   = 0x0c,
-	LIBHAL_DRIVE_TYPE_FLASHKEY              = 0x0d
+	LIBHAL_DRIVE_TYPE_FLASHKEY              = 0x0d,
+	LIBHAL_DRIVE_TYPE_MO                    = 0x0e
 } LibHalDriveType;
 
 typedef enum {
-	LIBHAL_DRIVE_CDROM_CAPS_CDROM      = 0x0001,
-	LIBHAL_DRIVE_CDROM_CAPS_CDR        = 0x0002,
-	LIBHAL_DRIVE_CDROM_CAPS_CDRW       = 0x0004,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDRAM     = 0x0008,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDROM     = 0x0010,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDR       = 0x0020,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDRW      = 0x0040,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR   = 0x0080,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW  = 0x0100,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL = 0x0200,
-	LIBHAL_DRIVE_CDROM_CAPS_BDROM      = 0x0400,
-	LIBHAL_DRIVE_CDROM_CAPS_BDR        = 0x0800,
-	LIBHAL_DRIVE_CDROM_CAPS_BDRE       = 0x1000,
-	LIBHAL_DRIVE_CDROM_CAPS_HDDVDROM   = 0x2000,
-	LIBHAL_DRIVE_CDROM_CAPS_HDDVDR     = 0x4000,
-	LIBHAL_DRIVE_CDROM_CAPS_HDDVDRW    = 0x8000
+	LIBHAL_DRIVE_CDROM_CAPS_CDROM       = 0x00001,
+	LIBHAL_DRIVE_CDROM_CAPS_CDR         = 0x00002,
+	LIBHAL_DRIVE_CDROM_CAPS_CDRW        = 0x00004,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDRAM      = 0x00008,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDROM      = 0x00010,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDR        = 0x00020,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDRW       = 0x00040,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR    = 0x00080,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW   = 0x00100,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL  = 0x00200,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRWDL = 0x00400,
+	LIBHAL_DRIVE_CDROM_CAPS_BDROM       = 0x00800,
+	LIBHAL_DRIVE_CDROM_CAPS_BDR         = 0x01000,
+	LIBHAL_DRIVE_CDROM_CAPS_BDRE        = 0x02000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDROM    = 0x04000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDR      = 0x08000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDRW     = 0x10000,
+	LIBHAL_DRIVE_CDROM_CAPS_MRW         = 0x20000,
+	LIBHAL_DRIVE_CDROM_CAPS_MRWW        = 0x40000,
+	LIBHAL_DRIVE_CDROM_CAPS_MO          = 0x80000
 } LibHalDriveCdromCaps;
 
 LibHalDrive         *libhal_drive_from_udi                    (LibHalContext *hal_ctx, 
@@ -201,6 +204,10 @@ void                 libhal_drive_free                        (LibHalDrive *driv
 
 dbus_bool_t          libhal_drive_is_hotpluggable          (LibHalDrive      *drive);
 dbus_bool_t          libhal_drive_uses_removable_media     (LibHalDrive      *drive);
+dbus_bool_t          libhal_drive_is_media_detected        (LibHalDrive      *drive);
+dbus_uint64_t        libhal_drive_get_size                 (LibHalDrive      *drive);
+dbus_uint64_t        libhal_drive_get_media_size           (LibHalDrive      *drive);
+const char          *libhal_drive_get_partition_scheme     (LibHalDrive      *drive);
 dbus_bool_t          libhal_drive_no_partitions_hint       (LibHalDrive      *drive);
 dbus_bool_t          libhal_drive_requires_eject           (LibHalDrive      *drive);
 LibHalDriveType      libhal_drive_get_type                 (LibHalDrive      *drive);
@@ -222,29 +229,29 @@ const char          *libhal_drive_get_dedicated_icon_volume   (LibHalDrive
 
 char                *libhal_drive_policy_compute_display_name (LibHalDrive        *drive, 
 							       LibHalVolume        *volume, 
-							       LibHalStoragePolicy *policy);
+							       LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 char                *libhal_drive_policy_compute_icon_name    (LibHalDrive         *drive, 
 							       LibHalVolume        *volume, 
-							       LibHalStoragePolicy *policy);
+							       LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 
 dbus_bool_t          libhal_drive_policy_is_mountable            (LibHalDrive         *drive, 
-								  LibHalStoragePolicy *policy);
+								  LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char          *libhal_drive_policy_get_desired_mount_point (LibHalDrive         *drive, 
-								  LibHalStoragePolicy *policy);
+								  LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char          *libhal_drive_policy_get_mount_options       (LibHalDrive         *drive, 
-							          LibHalStoragePolicy *policy);
+							          LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char          *libhal_drive_policy_get_mount_fs            (LibHalDrive      *drive, 
-								  LibHalStoragePolicy *policy);
+								  LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 
 char               **libhal_drive_find_all_volumes (LibHalContext *hal_ctx, 
 						    LibHalDrive   *drive,
 						    int 	  *num_volumes);
 
 
-char        *libhal_drive_policy_default_get_mount_root                (LibHalContext *hal_ctx);
-dbus_bool_t  libhal_drive_policy_default_use_managed_keyword           (LibHalContext *hal_ctx);
-char        *libhal_drive_policy_default_get_managed_keyword_primary   (LibHalContext *hal_ctx);
-char        *libhal_drive_policy_default_get_managed_keyword_secondary (LibHalContext *hal_ctx);
+char        *libhal_drive_policy_default_get_mount_root                (LibHalContext *hal_ctx) LIBHAL_DEPRECATED;
+dbus_bool_t  libhal_drive_policy_default_use_managed_keyword           (LibHalContext *hal_ctx) LIBHAL_DEPRECATED;
+char        *libhal_drive_policy_default_get_managed_keyword_primary   (LibHalContext *hal_ctx) LIBHAL_DEPRECATED;
+char        *libhal_drive_policy_default_get_managed_keyword_secondary (LibHalContext *hal_ctx) LIBHAL_DEPRECATED;
 
 
 typedef enum {
@@ -252,7 +259,8 @@ typedef enum {
 	LIBHAL_VOLUME_USAGE_PARTITION_TABLE,
 	LIBHAL_VOLUME_USAGE_RAID_MEMBER,
 	LIBHAL_VOLUME_USAGE_CRYPTO,
-	LIBHAL_VOLUME_USAGE_UNKNOWN
+	LIBHAL_VOLUME_USAGE_UNKNOWN,
+	LIBHAL_VOLUME_USAGE_OTHER
 } LibHalVolumeUsage;
 
 typedef enum {
@@ -271,15 +279,19 @@ typedef enum {
 	LIBHAL_VOLUME_DISC_TYPE_BDRE        = 0x0c,
 	LIBHAL_VOLUME_DISC_TYPE_HDDVDROM    = 0x0d,
 	LIBHAL_VOLUME_DISC_TYPE_HDDVDR      = 0x0e,
-	LIBHAL_VOLUME_DISC_TYPE_HDDVDRW     = 0x0f
+	LIBHAL_VOLUME_DISC_TYPE_HDDVDRW     = 0x0f,
+	LIBHAL_VOLUME_DISC_TYPE_MO          = 0x10,
 } LibHalVolumeDiscType;
 
 LibHalVolume     *libhal_volume_from_udi                      (LibHalContext *hal_ctx, 
 							       const char *udi);
 LibHalVolume     *libhal_volume_from_device_file              (LibHalContext *hal_ctx, 
 							       const char *device_file);
+LibHalVolume     *libhal_volume_from_mount_point              (LibHalContext *hal_ctx, 
+							       const char *mount_point);
 void              libhal_volume_free                          (LibHalVolume     *volume);
 dbus_uint64_t     libhal_volume_get_size                      (LibHalVolume     *volume);
+dbus_uint64_t     libhal_volume_get_disc_capacity             (LibHalVolume     *volume);
 
 const char          *libhal_volume_get_udi                       (LibHalVolume     *volume);
 const char          *libhal_volume_get_device_file               (LibHalVolume     *volume);
@@ -289,9 +301,19 @@ const char          *libhal_volume_get_fstype                    (LibHalVolume
 const char          *libhal_volume_get_fsversion                 (LibHalVolume     *volume);
 LibHalVolumeUsage    libhal_volume_get_fsusage                   (LibHalVolume     *volume);
 dbus_bool_t          libhal_volume_is_mounted                    (LibHalVolume     *volume);
+dbus_bool_t          libhal_volume_is_mounted_read_only          (LibHalVolume     *volume);
 dbus_bool_t          libhal_volume_is_partition                  (LibHalVolume     *volume);
 dbus_bool_t          libhal_volume_is_disc                       (LibHalVolume     *volume);
+
+const char          *libhal_volume_get_partition_scheme          (LibHalVolume     *volume);
+const char          *libhal_volume_get_partition_type            (LibHalVolume     *volume);
+const char          *libhal_volume_get_partition_label           (LibHalVolume     *volume);
+const char          *libhal_volume_get_partition_uuid            (LibHalVolume     *volume);
+const char         **libhal_volume_get_partition_flags           (LibHalVolume     *volume);
 unsigned int         libhal_volume_get_partition_number          (LibHalVolume     *volume);
+dbus_uint64_t        libhal_volume_get_partition_start_offset    (LibHalVolume     *volume);
+dbus_uint64_t        libhal_volume_get_partition_media_size      (LibHalVolume     *volume);
+
 const char          *libhal_volume_get_label                     (LibHalVolume     *volume);
 const char          *libhal_volume_get_mount_point               (LibHalVolume     *volume);
 const char          *libhal_volume_get_uuid                      (LibHalVolume     *volume);
@@ -308,38 +330,40 @@ dbus_bool_t          libhal_volume_disc_is_rewritable         (LibHalVolume
 dbus_bool_t          libhal_volume_disc_is_appendable         (LibHalVolume     *volume);
 LibHalVolumeDiscType libhal_volume_get_disc_type              (LibHalVolume     *volume);
 
-int               libhal_volume_get_msdos_part_table_type     (LibHalVolume     *volume);
+int               libhal_volume_get_msdos_part_table_type     (LibHalVolume     *volume) LIBHAL_DEPRECATED;
+dbus_uint64_t     libhal_volume_get_msdos_part_table_start    (LibHalVolume     *volume) LIBHAL_DEPRECATED;
+dbus_uint64_t     libhal_volume_get_msdos_part_table_size     (LibHalVolume     *volume) LIBHAL_DEPRECATED;
+	
 
 dbus_bool_t       libhal_volume_should_ignore 	              (LibHalVolume     *volume);
 
-char             *libhal_volume_policy_compute_size_as_string (LibHalVolume     *volume);
+char             *libhal_volume_policy_compute_size_as_string (LibHalVolume     *volume) LIBHAL_DEPRECATED;
 
 char             *libhal_volume_policy_compute_display_name   (LibHalDrive         *drive, 
 							       LibHalVolume        *volume, 
-							       LibHalStoragePolicy *policy);
+							       LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 char             *libhal_volume_policy_compute_icon_name      (LibHalDrive         *drive, 
 							       LibHalVolume        *volume, 
-							       LibHalStoragePolicy *policy);
+							       LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 
 dbus_bool_t       libhal_volume_policy_should_be_visible      (LibHalDrive         *drive, 
 							       LibHalVolume        *volume, 
 							       LibHalStoragePolicy *policy, 
-							       const char          *target_mount_point);
+							       const char          *target_mount_point) LIBHAL_DEPRECATED;
 
 dbus_bool_t       libhal_volume_policy_is_mountable		(LibHalDrive         *drive, 
 								 LibHalVolume        *volume, 
-								 LibHalStoragePolicy *policy);
+								 LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char       *libhal_volume_policy_get_desired_mount_point  (LibHalDrive         *drive, 
 								 LibHalVolume        *volume, 
-								 LibHalStoragePolicy *policy);
+								 LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char       *libhal_volume_policy_get_mount_options   	(LibHalDrive         *drive, 
 							    	 LibHalVolume        *volume, 
-							    	 LibHalStoragePolicy *policy);
+							    	 LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 const char       *libhal_volume_policy_get_mount_fs        	(LibHalDrive         *drive, 
 							    	 LibHalVolume        *volume, 
-							    	 LibHalStoragePolicy *policy);
+							    	 LibHalStoragePolicy *policy) LIBHAL_DEPRECATED;
 
-/** @} */
 
 #if defined(__cplusplus)
 }
