@@ -42,15 +42,17 @@ extern "C" {
 #endif 
 
 
+/**
+ * LIBHAL_LIBHAL_FREE_DBUS_ERROR:
+ * @_dbus_error_: the DBusError
+ *
+ * Handy macro for checking whether a DBusError is set and free the error if
+ * the error is set.
+ */
 #define LIBHAL_FREE_DBUS_ERROR(_dbus_error_)					\
 	do {									\
 		if (dbus_error_is_set(_dbus_error_))				\
 			dbus_error_free (_dbus_error_);				\
-		else								\
-			fprintf (stderr,					\
-				 "%s %d : INFO: called LIBHAL_FREE_DBUS_ERROR "	\
-				 "but dbusError was not set.\n", 		\
-				 __FILE__, __LINE__);				\
 	} while (0)
 
 
@@ -189,6 +191,67 @@ typedef void (*LibHalDeviceCondition) (LibHalContext *ctx,
 				       const char *condition_name,
 				       const char *condition_detail);
 
+/** 
+ * LibHalGlobalInterfaceLockAcquired:
+ * @ctx: context for connection to hald
+ * @interface_name: the name of the interface
+ * @lock_owner: what service acquired the lock
+ * @num_locks: number of locks on the interface
+ *
+ * Type for callback when someone acquires a global lock.
+ */
+typedef void (*LibHalGlobalInterfaceLockAcquired) (LibHalContext *ctx,
+                                                   const char *interface_name,
+                                                   const char *lock_owner,
+                                                   int         num_locks);
+
+/** 
+ * LibHalGlobalInterfaceLockReleased:
+ * @ctx: context for connection to hald
+ * @interface_name: the name of the interface
+ * @lock_owner: what service released the lock
+ * @num_locks: number of locks on the interface
+ *
+ * Type for callback when someone releases a global lock.
+ */
+typedef void (*LibHalGlobalInterfaceLockReleased) (LibHalContext *ctx,
+                                                   const char *interface_name,
+                                                   const char *lock_owner,
+                                                   int         num_locks);
+
+/** 
+ * LibHalInterfaceLockAcquired:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @interface_name: the name of the interface
+ * @lock_owner: what service acquired the lock
+ * @num_locks: number of locks on the interface
+ *
+ * Type for callback when someone acquires a lock on a device.
+ */
+typedef void (*LibHalInterfaceLockAcquired) (LibHalContext *ctx,
+                                             const char *udi,
+                                             const char *interface_name,
+                                             const char *lock_owner,
+                                             int         num_locks);
+
+/** 
+ * LibHalInterfaceLockReleased:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @interface_name: the name of the interface
+ * @lock_owner: what service released the lock
+ * @num_locks: number of locks on the interface
+ *
+ * Type for callback when someone acquires a lock on a device.
+ */
+typedef void (*LibHalInterfaceLockReleased) (LibHalContext *ctx,
+                                             const char *udi,
+                                             const char *interface_name,
+                                             const char *lock_owner,
+                                             int         num_locks);
+
+
 
 /* Create a new context for a connection with hald */
 LibHalContext *libhal_ctx_new                          (void);
@@ -226,6 +289,18 @@ dbus_bool_t    libhal_ctx_set_device_property_modified (LibHalContext *ctx, LibH
 /* Set the callback for when a device emits a condition */
 dbus_bool_t    libhal_ctx_set_device_condition         (LibHalContext *ctx, LibHalDeviceCondition callback);
 
+/* Set the callback for when a global interface lock is acquired  */
+dbus_bool_t    libhal_ctx_set_global_interface_lock_acquired (LibHalContext *ctx, LibHalGlobalInterfaceLockAcquired callback);
+
+/* Set the callback for when a global interface lock is released  */
+dbus_bool_t    libhal_ctx_set_global_interface_lock_released (LibHalContext *ctx, LibHalGlobalInterfaceLockReleased callback);
+
+/* Set the callback for when an interface lock is acquired  */
+dbus_bool_t    libhal_ctx_set_interface_lock_acquired (LibHalContext *ctx, LibHalInterfaceLockAcquired callback);
+
+/* Set the callback for when an interface lock is released  */
+dbus_bool_t    libhal_ctx_set_interface_lock_released (LibHalContext *ctx, LibHalInterfaceLockReleased callback);
+
 /* Initialize the connection to hald */
 dbus_bool_t    libhal_ctx_init                         (LibHalContext *ctx, DBusError *error);
 
@@ -412,6 +487,10 @@ LibHalPropertySet *libhal_device_get_all_properties (LibHalContext *ctx,
 						     const char *udi,
 						     DBusError *error);
 
+
+/* sort all properties according to property name */
+void libhal_property_set_sort (LibHalPropertySet *set);
+
 /* Free a property set earlier obtained with libhal_device_get_all_properties(). */
 void libhal_free_property_set (LibHalPropertySet *set);
 
@@ -597,6 +676,44 @@ dbus_bool_t libhal_device_claim_interface (LibHalContext *ctx,
 dbus_bool_t libhal_device_addon_is_ready (LibHalContext *ctx, const char *udi, DBusError *error);
 
 
+/* Take a mandatory lock on an interface on a device. */
+dbus_bool_t libhal_device_acquire_interface_lock (LibHalContext *ctx,
+                                                  const char *udi,
+                                                  const char *interface,
+                                                  dbus_bool_t exclusive,
+                                                  DBusError *error);
+
+/* Release a mandatory lock on an interface on a device. */
+dbus_bool_t libhal_device_release_interface_lock (LibHalContext *ctx,
+                                                  const char *udi,
+                                                  const char *interface,
+                                                  DBusError *error);
+
+/* Take a mandatory lock on an interface (the lock affects all devices the caller have access to). */
+dbus_bool_t libhal_acquire_global_interface_lock (LibHalContext *ctx,
+                                                  const char *interface,
+                                                  dbus_bool_t exclusive,
+                                                  DBusError *error);
+
+/* Release a mandatory lock on an interface (affects all devices the caller have access to). */
+dbus_bool_t libhal_release_global_interface_lock (LibHalContext *ctx,
+                                                  const char *interface,
+                                                  DBusError *error);
+
+/* Determine if a given caller is locked out of a given interface on a given device */
+dbus_bool_t libhal_device_is_caller_locked_out (LibHalContext *ctx,
+                                                const char *udi,
+                                                const char *interface,
+                                                const char *caller,
+                                                DBusError *error);
+
+/* Determines whether a determines other processes than the caller holds a lock on the given device.  */
+dbus_bool_t libhal_device_is_locked_by_others (LibHalContext *ctx,
+                                               const char *udi,
+                                               const char *interface,
+                                               DBusError *error);
+
+
 #if defined(__cplusplus)
 }
 #endif
