@@ -29,8 +29,12 @@ class DeviceManager(LibGladeApplication):
 
     def on_about_activate(self, w):
         """Show the about dialog."""
-        gnome.ui.About(Const.NAME_LONG, Const.VERSION, Const.COPYRIGHT,
-                       Const.INFO, Const.AUTHORS).show()
+    	try:
+            self.about.destroy()
+        finally:
+            self.about = gnome.ui.About(Const.NAME_LONG, Const.VERSION, Const.COPYRIGHT,
+                                        Const.INFO, Const.AUTHORS).show()
+            self.about.show()
 
     def on_virtual_devices_activate(self, obj):
         self.dont_show_virtual = 1 - self.dont_show_virtual
@@ -292,15 +296,21 @@ class DeviceManager(LibGladeApplication):
         category = self.xml.get_widget("ns_device_category")
         capabilities = self.xml.get_widget("ns_device_capabilities")
 
-	if not device.properties.has_key("info.bus"):
-            product.set_label("Unknown")
-            vendor.set_label("Unknown")
-	else:
-	    bus.set_label(Const.BUS_NAMES[device.properties["info.bus"]])	    
+        product.set_label("Unknown")
+        vendor.set_label("Unknown")
         #state.set_label(Const.STATE_NAMES[device.properties["State"]])
 
+        # walk up the device tree to find the bus type
+        bus.set_label("Unknown")
+        d = device
+        while d:
+            if d.properties.has_key("info.subsystem"):
+                bus.set_label(Const.BUS_NAMES[d.properties["info.subsystem"]])
+                break
+            d = self.udi_to_device(d.properties["info.parent"])
+
         # guestimate product and vendor if we have no device information file
-        if device.properties.has_key("info.bus") and device.properties["info.bus"]=="usb":
+        if device.properties.has_key("info.subsystem") and device.properties["info.subsystem"]=="usb":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             elif device.properties.has_key("usb.product"):
@@ -320,7 +330,7 @@ class DeviceManager(LibGladeApplication):
                 vendor.set_label("Unknown")
 
 
-        elif device.properties.has_key("info.bus") and device.properties["info.bus"]=="pci":
+        elif device.properties.has_key("info.subsystem") and device.properties["info.subsystem"]=="pci":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             elif device.properties.has_key("pci.product"):
@@ -338,7 +348,7 @@ class DeviceManager(LibGladeApplication):
                 vendor.set_label("Unknown (0x%x)"%device.properties["pci.vendor_id"])
             else:
                 vendor.set_label("Unknown")
-        elif device.properties.has_key("info.bus") and device.properties["info.bus"]=="block":
+        elif device.properties.has_key("info.subsystem") and device.properties["info.subsystem"]=="block":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             else:
@@ -360,14 +370,14 @@ class DeviceManager(LibGladeApplication):
 	    category.set_label("Unknown")
 
 	if device.properties.has_key("info.capabilities"):
-            capabilities.set_label("%s"%device.properties["info.capabilities"])
+            capabilities.set_label(", ".join(device.properties["info.capabilities"]))
 	else:
 	    capabilities.set_label("Unknown")
 
     def update_tab_usb(self, device):
         """Updates the 'USB' tab given a Device object; may hide it"""
         page = self.xml.get_widget("device_notebook").get_nth_page(1)
-        if not device.properties.has_key("info.bus") or device.properties["info.bus"]!="usb":
+        if not device.properties.has_key("info.subsystem") or device.properties["info.subsystem"]!="usb":
             page.hide_all()
             return
 
@@ -401,7 +411,7 @@ class DeviceManager(LibGladeApplication):
     def update_tab_pci(self, device):
         """Updates the 'PCI' tab given a Device object; may hide it"""
         page = self.xml.get_widget("device_notebook").get_nth_page(2)
-        if not device.properties.has_key("info.bus") or device.properties["info.bus"]!="pci":
+        if not device.properties.has_key("info.subsystem") or device.properties["info.subsystem"]!="pci":
             page.hide_all()
             return
 
@@ -454,6 +464,8 @@ class DeviceManager(LibGladeApplication):
                     store.set(iter, 0, p, 1, "bool", 2, "false")
             elif ptype==float:
                 store.set(iter, 0, p, 1, "float", 2, "%f"%val)
+	    elif ptype==list:
+		store.set(iter, 0, p, 1, "list", 2, ", ".join(val))
 	    else:
 		# assume strlist
 		store.set(iter, 0, p, 1, "strlist", 2, val)
