@@ -35,21 +35,14 @@
 
 #include <libhal.h>
 
-/**
- * @defgroup HalGetProperty  Get HAL device property
- * @ingroup HalMisc
- *
- * @brief A commandline tool getting a property of a device. Uses libhal
- *
- * @{
- */
-
 static LibHalContext *hal_ctx;
 
-/** Print out program usage.
+/** 
+ *  usage:
+ *  @argc:                Number of arguments given to program
+ *  @argv:                Arguments given to program
  *
- *  @param  argc                Number of arguments given to program
- *  @param  argv                Arguments given to program
+ *  Print out program usage. 
  */
 static void
 usage (int argc, char *argv[])
@@ -73,11 +66,14 @@ usage (int argc, char *argv[])
  "\n");
 }
 
-/** Entry point
+/** 
+ *  main:
+ *  @argc:                Number of arguments given to program
+ *  @argv:                Arguments given to program
  *
- *  @param  argc                Number of arguments given to program
- *  @param  argv                Arguments given to program
- *  @return                     Return code
+ *  Returns:              Return code
+ *
+ *  Main entry point 
  */
 int
 main (int argc, char *argv[])
@@ -163,8 +159,12 @@ main (int argc, char *argv[])
 		return 1;
 	}
 	if (!libhal_ctx_init (hal_ctx, &error)) {
-		fprintf (stderr, "error: libhal_ctx_init: %s: %s\n", error.name, error.message);
-		LIBHAL_FREE_DBUS_ERROR (&error);
+		if (dbus_error_is_set(&error)) {
+			fprintf (stderr, "error: libhal_ctx_init: %s: %s\n", error.name, error.message);
+			dbus_error_free (&error);
+		}
+		fprintf (stderr, "Could not initialise connection to hald.\n"
+				 "Normally this means the HAL daemon (hald) is not running or not ready.\n");
 		return 1;
 	}
 
@@ -240,7 +240,3 @@ main (int argc, char *argv[])
 
 	return 0;
 }
-
-/**
- * @}
- */
