@@ -34,6 +34,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <time.h>
+#include <sys/time.h>
 
 #include <glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -44,17 +46,7 @@
 #define __FUNCTION__ __func__
 #endif
 
-/**
- * @defgroup HalLsHal  List HAL devices
- * @ingroup HalMisc
- *
- * @brief A commandline tool, lshal, for displaying and, optionally,
- *        monitor the devices managed by the HAL daemon. Uses libhal.
- *
- * @{
- */
-
-/** Macro for terminating the program on an unrecoverable error */
+/* Macro for terminating the program on an unrecoverable error */
 #define DIE(expr) do {printf("*** [DIE] %s:%s():%d : ", __FILE__, __FUNCTION__, __LINE__); printf expr; printf("\n"); exit(1); } while(0)
 
 #define UDI_BASE "/org/freedesktop/Hal/devices/"
@@ -70,9 +62,13 @@ struct Device {
 	char *parent;
 };
 
-/** Generate a short name for a device
+/** 
+ *  short_name:
+ *  @udi:               Universal Device Id
+ * 
+ *  Returns:		short name of a device
  *
- *  @param  udi                 Universal Device Id
+ *  Generate a short name for a device 
  */
 static const char *
 short_name (const char *udi)
@@ -80,11 +76,31 @@ short_name (const char *udi)
 	return &udi[sizeof(UDI_BASE) - 1];
 }
 
-/** Print all properties of a device
- *
- *  @param  udi                 Universal Device Id
- */
+static char *
+get_time (void)
+{
+	struct timeval tnow;
+	struct tm *tlocaltime;
+	struct timezone tzone;
+	static char tbuf[256];
+	static char buf[256];
+	GTimeVal t;
 
+	gettimeofday (&tnow, &tzone);
+	tlocaltime = localtime ((time_t *) &tnow.tv_sec);
+	strftime (tbuf, sizeof (tbuf), "%H:%M:%S", tlocaltime);
+
+	g_get_current_time (&t);
+	g_snprintf (buf, sizeof(buf), "%s.%03d", tbuf, (int) (t.tv_usec / 1000));
+	return buf;
+}
+
+/** 
+ *  print_props:
+ *  @udi:                Universal Device Id
+ *
+ *  Print all properties of a device 
+ */
 static void
 print_props (const char *udi)
 {
@@ -108,6 +124,8 @@ print_props (const char *udi)
 		return;
 	}
 
+	libhal_property_set_sort (props);
+
 	for (libhal_psi_init (&it, props); libhal_psi_has_more (&it); libhal_psi_next (&it)) {
 		type = libhal_psi_get_type (&it);
 		switch (type) {
@@ -132,8 +150,9 @@ print_props (const char *udi)
 			break;
 
 		case LIBHAL_PROPERTY_TYPE_DOUBLE:
-			printf ("  %s = %g  (double)\n",
+			printf ("  %s = %.1f (%g) (double)\n",
 				libhal_psi_get_key (&it),
+				libhal_psi_get_double (&it),
 				libhal_psi_get_double (&it));
 			break;
 
@@ -170,11 +189,12 @@ print_props (const char *udi)
 	libhal_free_property_set (props);
 }
 
-/** Dumps information about a single device
+/** 
+ *  dump_device:
+ *  @udi:                 Universal Device Id
  *
- *  @param  udi                 Universal Device Id
+ *  Dumps information about a single device 
  */
-
 static void
 dump_device (const char *udi)
 {
@@ -197,14 +217,15 @@ dump_device (const char *udi)
 		printf ("%s\n", short_name (udi));
 }
 
-/** Dump all children of device
+/** 
+ *  dump_children:
+ *  @udi:                 Universal Device Id of parent
+ *  @num_devices:         Total number of devices in device list
+ *  @devices:             List of devices
+ *  @depth:               Current recursion depth
  *
- *  @param  udi                 Universal Device Id of parent
- *  @param  num_devices         Total number of devices in device list
- *  @param  devices             List of devices
- *  @param  depth               Current recursion depth
+ *  Dump all children of device 
  */
-
 static void
 dump_children (char *udi, int num_devices, struct Device *devices, int depth)
 {
@@ -242,8 +263,10 @@ dump_children (char *udi, int num_devices, struct Device *devices, int depth)
 	}
 }
 
-/** Dump all devices to stdout
- *
+/** 
+ *  dump_devices:
+ *  
+ *  Dump all devices to stdout
  */
 static void
 dump_devices (void)
@@ -307,10 +330,13 @@ dump_devices (void)
 	}
 }
 
-/** Invoked when a device is added to the Global Device List. Simply prints
- *  a message on stdout.
+/** 
+ *  device_added:
+ *  @ctx:		The HAL Context
+ *  @udi:                Universal Device Id
  *
- *  @param  udi                 Universal Device Id
+ *  Invoked when a device is added to the Global Device List. Simply prints
+ *  a message on stdout. 
  */
 static void
 device_added (LibHalContext *ctx,
@@ -320,16 +346,19 @@ device_added (LibHalContext *ctx,
 		return;
 
 	if (long_list) {
-		printf ("*** lshal: device_added, udi='%s'\n", udi);
+		printf ("*** %s: lshal: device_added, udi='%s'\n", get_time (), udi);
 		print_props (udi);
 	} else
-		printf ("%s added\n", short_name (udi));
+		printf ("%s: %s added\n", get_time (), short_name (udi));
 }
 
-/** Invoked when a device is removed from the Global Device List. Simply
- *  prints a message on stdout.
+/** 
+ *  device_removed:
+ *  @ctx:		The HAL Context
+ *  @udi:               Universal Device Id
  *
- *  @param  udi                 Universal Device Id
+ *  Invoked when a device is removed from the Global Device List. Simply
+ *  prints a message on stdout. 
  */
 static void
 device_removed (LibHalContext *ctx,
@@ -339,16 +368,19 @@ device_removed (LibHalContext *ctx,
 		return;
 
 	if (long_list)
-		printf ("*** lshal: device_removed, udi='%s'\n", udi);
+		printf ("*** %s: lshal: device_removed, udi='%s'\n", get_time (), udi);
 	else
-		printf ("%s removed\n", short_name (udi));
+		printf ("%s: %s removed\n", get_time (), short_name (udi));
 }
 
-/** Invoked when device in the Global Device List acquires a new capability.
- *  Prints the name of the capability to stdout.
+/** 
+ *  device_new_capability:
+ *  @ctx:		The HAL Context
+ *  @udi:               Universal Device Id
+ *  @capability:        Name of capability
  *
- *  @param  udi                 Universal Device Id
- *  @param  capability          Name of capability
+ *  Invoked when device in the Global Device List acquires a new capability.
+ *  Prints the name of the capability to stdout. 
  */
 static void
 device_new_capability (LibHalContext *ctx,
@@ -359,18 +391,21 @@ device_new_capability (LibHalContext *ctx,
 		return;
 
 	if (long_list) {
-		printf ("*** lshal: new_capability, udi='%s'\n", udi);
+		printf ("*** %s: lshal: new_capability, udi='%s'\n", get_time (), udi);
 		printf ("*** capability: %s\n", capability);
 	} else
-		printf ("%s capability %s added\n", short_name (udi),
+		printf ("%s: %s capability %s added\n", get_time (), short_name (udi),
 			capability);
 }
 
-/** Invoked when device in the Global Device List loses a capability.
- *  Prints the name of the capability to stdout.
- *
- *  @param  udi                 Universal Device Id
- *  @param  capability          Name of capability
+/** 
+ *  device_lost_capability:
+ *  @ctx:		The HAL Context
+ *  @udi:               Universal Device Id
+ *  @capability:        Name of capability
+ * 
+ *  Invoked when device in the Global Device List loses a capability.
+ *  Prints the name of the capability to stdout. 
  */
 static void
 device_lost_capability (LibHalContext *ctx,
@@ -381,17 +416,19 @@ device_lost_capability (LibHalContext *ctx,
 		return;
 
 	if (long_list) {
-		printf ("*** lshal: lost_capability, udi='%s'\n", udi);
+		printf ("*** %s: lshal: lost_capability, udi='%s'\n", get_time (), udi);
 		printf ("*** capability: %s\n", capability);
 	} else
-		printf ("%s capability %s lost\n", short_name (udi),
+		printf ("%s: %s capability %s lost\n", get_time (), short_name (udi),
 			capability);
 }
 
-/** Acquires and prints the value of of a property to stdout.
+/** 
+ *  print_property:
+ *  @udi:                 Universal Device Id
+ *  @key:                 Key of property
  *
- *  @param  udi                 Universal Device Id
- *  @param  key                 Key of property
+ *  Acquires and prints the value of of a property to stdout. 
  */
 static void
 print_property (const char *udi, const char *key)
@@ -443,16 +480,19 @@ print_property (const char *udi, const char *key)
 			printf ("{");
 
 		strlist = libhal_device_get_property_strlist (hal_ctx, udi, key, &error);
-		for (i = 0; strlist[i] != 0; i++) {
-			printf ("'%s'", strlist[i]);
-			if (strlist[i+1] != NULL)
-				printf (", ");
-		}
-		if (long_list)
-			printf ("}  (string list)\n");
-		else
-			printf ("}");
-		libhal_free_string_array (strlist);
+                /* may be NULL because property may have been removed */
+                if (strlist != NULL) {
+                        for (i = 0; strlist[i] != 0; i++) {
+                                printf ("'%s'", strlist[i]);
+                                if (strlist[i+1] != NULL)
+                                        printf (", ");
+                        }
+                        if (long_list)
+                                printf ("}  (string list)\n");
+                        else
+                                printf ("}");
+                        libhal_free_string_array (strlist);
+                }
 		break;
 	}
 
@@ -465,11 +505,16 @@ print_property (const char *udi, const char *key)
 		dbus_error_free (&error);
 }
 
-/** Invoked when a property of a device in the Global Device List is
- *  changed, and we have we have subscribed to changes for that device.
- *
- *  @param  udi                 Univerisal Device Id
- *  @param  key                 Key of property
+/** 
+ *  property_modified:
+ *  @ctx:		The HAL Context
+ *  @udi:               Univerisal Device Id
+ *  @key:               Key of property
+ *  @is_removed:        if the property was removed
+ *  @is_added:          if the property was added
+ * 
+ *  Invoked when a property of a device in the Global Device List is
+ *  changed, and we have we have subscribed to changes for that device. 
  */
 static void
 property_modified (LibHalContext *ctx,
@@ -482,8 +527,8 @@ property_modified (LibHalContext *ctx,
 		return;
 
 	if (long_list) {
-		printf ("*** lshal: property_modified, udi=%s, key=%s\n",
-			udi, key);
+		printf ("*** %s: lshal: property_modified, udi=%s, key=%s\n",
+			get_time (), udi, key);
 		printf ("           is_removed=%s, is_added=%s\n",
 			is_removed ? "true" : "false",
 			is_added ? "true" : "false");
@@ -491,7 +536,7 @@ property_modified (LibHalContext *ctx,
 			print_property (udi, key);
 		printf ("\n");
 	} else {
-		printf ("%s property %s ", short_name (udi), key);
+		printf ("%s: %s property %s ", get_time (), short_name (udi), key);
 		if (is_removed)
 			printf ("removed");
 		else {
@@ -506,12 +551,15 @@ property_modified (LibHalContext *ctx,
 }
 
 
-/** Invoked when a property of a device in the Global Device List is
- *  changed, and we have we have subscribed to changes for that device.
+/** 
+ *  device_condition:
+ *  @ctx:                 The HAL Context
+ *  @udi:                 Univerisal Device Id
+ *  @condition_name:      Name of condition
+ *  @message:             D-BUS message with parameters
  *
- *  @param  udi                 Univerisal Device Id
- *  @param  condition_name      Name of condition
- *  @param  message             D-BUS message with parameters
+ *  Invoked when a property of a device in the Global Device List is
+ *  changed, and we have we have subscribed to changes for that device. 
  */
 static void
 device_condition (LibHalContext *ctx,
@@ -523,21 +571,94 @@ device_condition (LibHalContext *ctx,
 		return;
 
 	if (long_list) {
-		printf ("*** lshal: device_condition, udi=%s\n", udi);
+		printf ("*** %s: lshal: device_condition, udi=%s\n", get_time (), udi);
 		printf ("           condition_name=%s\n", condition_name);
 		printf ("           condition_details=%s\n", condition_details);
 		printf ("\n");
 	} else {
-		printf ("%s condition %s = %s\n", short_name (udi),
+		printf ("%s: %s condition %s = %s\n", get_time (), short_name (udi),
 			condition_name, condition_details);
 	}
 }
 
+static void
+do_interface_lock (LibHalContext *ctx,
+                   dbus_bool_t acquired,
+                   dbus_bool_t global,
+                   const char *udi,
+                   const char *interface_name,
+                   const char *lock_owner,
+                   int num_locks)
+{
+	if (show_device && strcmp(show_device, udi))
+		return;
 
-/** Print out program usage.
+	if (long_list) {
+                if (global)
+                        printf ("*** %s: lshal: global_interface_lock_%s\n", get_time (), acquired ? "acquired" : "released");
+                else
+                        printf ("*** %s: lshal: interface_lock_%s, udi=%s\n", get_time (), acquired ? "acquired" : "released", udi);
+		printf ("           interface_name=%s\n", interface_name);
+		printf ("           lock_owner=%s\n", lock_owner);
+		printf ("           num_locks=%d\n", num_locks);
+		printf ("\n");
+	} else {
+                if (global)
+                        printf ("%s: global_interface_lock_%s %s by %s (%d lockers)\n", get_time (),
+                                acquired ? "acquired" : "released",
+                                interface_name, lock_owner, num_locks);
+                else
+                        printf ("%s: %s interface_lock_%s %s by %s (%d lockers)\n", get_time (), short_name (udi),
+                                acquired ? "acquired" : "released",
+                                interface_name, lock_owner, num_locks);
+	}
+}
+
+static void
+global_interface_lock_acquired (LibHalContext *ctx,
+                                const char *interface_name,
+                                const char *lock_owner,
+                                int num_locks)
+{
+        do_interface_lock (ctx, TRUE, TRUE, NULL, interface_name, lock_owner, num_locks);
+}
+
+static void
+global_interface_lock_released (LibHalContext *ctx,
+                                const char *interface_name,
+                                const char *lock_owner,
+                                int num_locks)
+{
+        do_interface_lock (ctx, FALSE, TRUE, NULL, interface_name, lock_owner, num_locks);
+}
+
+static void
+interface_lock_acquired (LibHalContext *ctx,
+                         const char *udi,
+                         const char *interface_name,
+                         const char *lock_owner,
+                         int num_locks)
+{
+        do_interface_lock (ctx, TRUE, FALSE, udi, interface_name, lock_owner, num_locks);
+}
+
+static void
+interface_lock_released (LibHalContext *ctx,
+                         const char *udi,
+                         const char *interface_name,
+                         const char *lock_owner,
+                         int num_locks)
+{
+        do_interface_lock (ctx, FALSE, FALSE, udi, interface_name, lock_owner, num_locks);
+}
+
+
+/** 
+ *  usage:
+ *  @argc:                Number of arguments given to program
+ *  @argv:                Arguments given to program
  *
- *  @param  argc                Number of arguments given to program
- *  @param  argv                Arguments given to program
+ *  Print out program usage. 
  */
 static void
 usage (int argc, char *argv[])
@@ -564,11 +685,14 @@ usage (int argc, char *argv[])
 		 "\n");
 }
 
-/** Entry point
+/** 
+ *  main:
+ *  @argc:                Number of arguments given to program
+ *  @argv:                Arguments given to program
  *
- *  @param  argc                Number of arguments given to program
- *  @param  argv                Arguments given to program
- *  @return                     Return code
+ *  Returns:              Return code
+ *
+ *  Main entry point 
  */
 int
 main (int argc, char *argv[])
@@ -682,11 +806,12 @@ main (int argc, char *argv[])
 		return 1;
 	}
 	if (!libhal_ctx_init (hal_ctx, &error)) {
-		fprintf (stderr, "error: libhal_ctx_init: %s: %s\n",
-			 error.name, error.message);
-		fprintf (stderr, "Could not initialise connection to hald. \n "
-				 "Normally this mean the HAL daemon (hald) is not running or not ready.\n");
-		LIBHAL_FREE_DBUS_ERROR (&error);
+		if (dbus_error_is_set(&error)) {
+			fprintf (stderr, "error: libhal_ctx_init: %s: %s\n", error.name, error.message);
+			dbus_error_free (&error);
+		}
+		fprintf (stderr, "Could not initialise connection to hald.\n"
+				 "Normally this means the HAL daemon (hald) is not running or not ready.\n");
 		return 1;
 	}
 
@@ -696,6 +821,10 @@ main (int argc, char *argv[])
 	libhal_ctx_set_device_lost_capability (hal_ctx, device_lost_capability);
 	libhal_ctx_set_device_property_modified (hal_ctx, property_modified);
 	libhal_ctx_set_device_condition (hal_ctx, device_condition);
+	libhal_ctx_set_global_interface_lock_acquired (hal_ctx, global_interface_lock_acquired);
+	libhal_ctx_set_global_interface_lock_released (hal_ctx, global_interface_lock_released);
+	libhal_ctx_set_interface_lock_acquired (hal_ctx, interface_lock_acquired);
+	libhal_ctx_set_interface_lock_released (hal_ctx, interface_lock_released);
 
 	if (show_device)
 		dump_device (show_device);
@@ -722,7 +851,6 @@ main (int argc, char *argv[])
 		LIBHAL_FREE_DBUS_ERROR (&error);
 	libhal_ctx_free (hal_ctx);
 
-	dbus_connection_close (conn);
 	dbus_connection_unref (conn);
 
 	if (show_device)
@@ -731,6 +859,3 @@ main (int argc, char *argv[])
 	return 0;
 }
 
-/**
- * @}
- */
